# CSC 240, Homework 7: Texture Mapping

# TODO 0: Name


import math
import sys
from OpenGL.GL import *
from OpenGL.GLU import *
from OpenGL.GLUT import *

from PIL import Image


# Handles the keyboard event
def special(key, x, y):
    # TODO: implement rotation around y (left and right arrows)
    # and x (up and down arrows)
    
    glutPostRedisplay()


# given a filename (string), set up and return the texture
def texture_from_jpg(filename):
    img = Image.open(filename)
    img_data = list(img.getdata())

    glPixelStorei(GL_UNPACK_ALIGNMENT,1)

    tex_name = glGenTextures(1)
    glBindTexture(GL_TEXTURE_2D, tex_name)

    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP)
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP)
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR)
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR)
    glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, img.size[0], img.size[1], 0, GL_RGB, GL_UNSIGNED_BYTE, img_data)

    return tex_name


# TODO: implement this method, which should return the UV coordiates
# for the texture, given a point on the sphere
def uv_coords(dx,dy,dz):
    pass
    

# create a circular table for use in glutSolidSphere
def circleTable(n):

    # Table size, the sign of n flips the circle direction
    size = abs(n)

    # Determine the angle between samples
    angle = 2*math.pi/n

    # Allocate memory for n samples, plus duplicate of first entry at the end
    sint = [0]*(size+1)
    cost = [0]*(size+1)

    # Compute cos and sin around the circle
    sint[0] = 0
    cost[0] = 1

    for i in range(1,size):
        sint[i] = math.sin(angle*i)
        cost[i] = math.cos(angle*i)

    # Last sample is duplicate of the first
    sint[size] = sint[0]
    cost[size] = cost[0]

    return sint,cost


# Source code for glutSolidSphere (based on http://freeglut.sourcearchive.com/)
def myGlutSolidSphere(radius, slices, stacks):

    # precompute sin/cos values
    sint1, cost1 = circleTable(-slices)
    sint2, cost2 = circleTable(stacks*2)

    # The top stack is covered with a triangle fan 
    y0 = 1
    y1 = cost2[1]
    r0 = 0
    r1 = sint2[1]

    glBegin(GL_TRIANGLE_FAN)

    glNormal3d(0,1,0)
    glVertex3d(0,radius,0)

    for j in range(slices,-1,-1):
        glNormal3d(cost1[j]*r1, y1, sint1[j]*r1)
        glVertex3d(cost1[j]*r1*radius, y1*radius, sint1[j]*r1*radius)

    glEnd()

    # Cover each stack with a quad strip, except the top and bottom stacks
    for i in range(1,stacks-1):
    
        y0 = y1
        y1 = cost2[i+1]
        r0 = r1
        r1 = sint2[i+1]

        glBegin(GL_QUAD_STRIP)

        for j in range(slices+1):
            
            glNormal3d(cost1[j]*r1, y1, sint1[j]*r1)
            glVertex3d(cost1[j]*r1*radius, y1*radius, sint1[j]*r1*radius)
            glNormal3d(cost1[j]*r0, y0, sint1[j]*r0)
            glVertex3d(cost1[j]*r0*radius, y0*radius, sint1[j]*r0*radius)

        glEnd()
    
    # The bottom stack is covered with a triangle fan
    y0 = y1
    r0 = r1

    glBegin(GL_TRIANGLE_FAN)

    glNormal3d(0,-1,0)
    glVertex3d(0,-radius,0)

    for j in range(slices+1):
        glNormal3d(cost1[j]*r0, y0, sint1[j]*r0)
        glVertex3d(cost1[j]*r0*radius, y0*radius, sint1[j]*r0*radius)

    glEnd()

  
def display():
    glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)

    glEnable(GL_TEXTURE_2D)
    glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL)

    glMatrixMode(GL_MODELVIEW)

    # lighting (temporary just to see the sphere, will be overwritten by the texture)
    ambient_color = [ 0.2, 0.2, 0.2, 1]
    light_position = [1,1,1,0]
    glMaterialfv(GL_FRONT, GL_AMBIENT, ambient_color)
    glLightfv(GL_LIGHT0, GL_POSITION, light_position)

    # draw the sphere
    myGlutSolidSphere(1,40,40)

    glFlush()
    glDisable(GL_TEXTURE_2D)


# Handles the reshape event by setting the viewport so that it takes up the
# whole visible region, then sets the projection matrix to something reason-
# able that maintains proper aspect ratio.
def reshape(w, h):
    glViewport(0, 0, w, h)
    glMatrixMode(GL_PROJECTION)
    glLoadIdentity()
    gluPerspective(60, w/ h, 1, 30)
    glMatrixMode(GL_MODELVIEW)
    glLoadIdentity()
    glTranslatef(0, 0, -3.6)


# Perfroms application specific initialization.
def init():

    glEnable(GL_LIGHTING)
    glEnable(GL_LIGHT0)
    glClearColor(0, 0, 0, 0)
    glShadeModel(GL_FLAT)
    glEnable(GL_DEPTH_TEST)

    # set up the texture here (TODO uncomment)
    #global worldmap
    #world = texture_from_jpg("worldmap.jpg")
  


# Initializes GLUT, the display mode, and main window; registers callbacks;
# does application initialization; enters the main event loop.
if __name__=='__main__':
    glutInit(sys.argv)
    glutInitDisplayMode(GLUT_SINGLE | GLUT_RGB | GLUT_DEPTH)
    glutInitWindowPosition(80, 80)
    glutInitWindowSize(800, 800)
    glutCreateWindow(b"Globe")
    init()
    glutDisplayFunc(display)
    glutReshapeFunc(reshape)
    glutSpecialFunc(special)
    glutMainLoop()

